//	DOOM system stuff for Cisco SX80

#include <stdlib.h>
#include <stdarg.h>
#include <sys/time.h>
#include <ctype.h>
#include <time.h>
#include <stdint.h>
#include <unistd.h>
#include <fcntl.h>

#include <tmc/sync.h>
#include <arch/cycle.h>

#include "doomdef.h"
#include "m_misc.h"
#include "i_video.h"
#include "i_sound.h"

#include "d_net.h"
#include "g_game.h"

#ifdef __GNUG__
#pragma implementation "i_system.h"
#endif
#include "i_system.h"




int	mb_used = 6;


int I_strncasecmp(char *str1, char *str2, int len)
{
	char c1, c2;

	while ( *str1 && *str2 && len-- ) {
		c1 = *str1++;
		c2 = *str2++;
		if ( toupper(c1) != toupper(c2) )
			return(1);
	}
	return(0);
}

void
I_Tactile
( int	on,
  int	off,
  int	total )
{
  // UNUSED.
  on = off = total = 0;
}

ticcmd_t	emptycmd;
ticcmd_t*	I_BaseTiccmd(void)
{
    return &emptycmd;
}


int  I_GetHeapSize (void)
{
    return mb_used*1024*1024;
}

byte* I_ZoneBase (int*	size)
{
    *size = mb_used*1024*1024;
    return (byte *) malloc (*size);
}


#define CPUFREQ 1200000000

static uint64_t basetime = 0;

//
// I_GetTime
// returns time in 1/35 second tics
//
int  I_GetTime (void)
{
    return (get_cycle_count()-basetime)*TICRATE/CPUFREQ;
}



//
// I_Init
//
void I_Init (void)
{
    basetime = get_cycle_count();

    I_InitSound();
    //  I_InitGraphics();
}

//
// I_Quit
//
void I_Quit (void)
{
    D_QuitNetGame ();
    I_ShutdownSound();
    I_ShutdownMusic();
    M_SaveDefaults ();

    I_ShutdownGraphics();

    exit(0);
}

extern tmc_sync_barrier_t *bar;
extern int measure;

void I_WaitVBL(int count)
{
//  usleep (count * (1000000/70) );
    int start = I_GetTime();
    while( true ) {
        if( !measure )
            tmc_sync_barrier_wait( bar );
        int now = I_GetTime();
        if( (now-start)*2 >= count ) break;
    }
}

void I_BeginRead(void)
{
}

void I_EndRead(void)
{
}

byte*	I_AllocLow(int length)
{
    byte*	mem;

    mem = (byte *)malloc (length);
    memset (mem,0,length);
    return mem;
}


//
// I_Error
//
extern boolean demorecording;

void I_Error (char *error, ...)
{
    va_list	argptr;

    // Message first.
    va_start (argptr,error);
    fprintf (stderr, "Error: ");
    vfprintf (stderr,error,argptr);
    fprintf (stderr, "\n");
    va_end (argptr);

    fflush( stderr );

    // Shutdown. Here might be other errors.
    if (demorecording)
	G_CheckDemoStatus();

    D_QuitNetGame ();
    I_ShutdownGraphics();

    exit(-1);
}
