//	DOOM graphics stuff for Cisco SX80

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdint.h>
#include <fcntl.h>
#include <linux/input.h>

#include <postinc.h>
#include <tmc/sync.h>

#include "m_swap.h"
#include "doomstat.h"
#include "i_system.h"
#include "v_video.h"
#include "m_argv.h"
#include "d_main.h"

#include "doomdef.h"


extern int measure;
extern tmc_sync_barrier_t *bar;
extern uint32_t *myscreen;
extern int mystride;
extern int coreid;
extern int ir;
extern int kb;

static uint32_t pal32[256];
//#define RENDER(R,G,B,A) (((A)<<24) | ((R)<<16) | ((G)<<8) | ((B)<<0))

// Fake mouse handling.
boolean	grabMouse;

// Blocky mode,
// replace each 320x200 pixel with multiply*multiply pixels.
// According to Dave Taylor, it still is a bonehead thing
// to use ....
int multiply = 1;


//
// Translates the key
//

static int xlatekey( int sym )
{
    int rc;

    // ir only remap
    switch( sym ) {
    case  1: rc = KEY_1;          break;
    case  2: rc = KEY_2;          break;
    case  3: rc = KEY_3;          break;
    case  4: rc = KEY_4;          break;
    case  5: rc = KEY_5;          break;
    case  6: rc = KEY_6;          break;
    case  7: rc = KEY_7;          break;
    case  8: rc = KEY_8;          break;
    case  9: rc = KEY_9;          break;
    case 10: rc = KEY_0;          break;
    case 11: rc = KEY_Y;          break; // *
    case 12: rc = KEY_N;          break; // #
    case 31: rc = KEY_LEFTARROW;  break;
    case 32: rc = KEY_RIGHTARROW; break;
    case 30: rc = KEY_DOWNARROW;  break;
    case 29: rc = KEY_UPARROW;    break;
    case 33: rc = KEY_ENTER;      break; // OK
    case 36: rc = KEY_TAB;        break; // Selfview
    case 37: rc = KEY_RCTRL;      break; // Layout
    case 38: rc = KEY_ESC;        break; // Cancel
    case 40: rc = KEY_SPACE;      break; // Directory
    case 49: rc = KEY_F11;        break; // Presets
    default: rc = 0;              break;
    }

    return rc;
}

void I_ShutdownGraphics( void )
{
}



//
// I_StartFrame
//
void I_StartFrame( void )
{
    // er?
}

static int do_kb( void )
{
    int rc;
    struct input_event ev;
    event_t event;

    rc = read( kb, &ev, sizeof(ev) );
    if( rc != sizeof(ev) ) return 0;

    event.data1 = ev.code;
    event.type  = ev.value == 0 ? ev_keyup : ev_keydown;
    D_PostEvent( &event );

    return 1;
}

int currkey = 0;
int downtime = 0;

static int do_ir( void )
{
    int rc;
    struct input_event ev;
    event_t event;
    int doomkey;

    // any keys waiting?
    rc = read( ir, &ev, sizeof(ev) );
    if( rc != sizeof(ev) ) {
        if( downtime != 0 && I_GetTime() >= downtime + 1 ) {
            event.type  = ev_keyup;
            event.data1 = currkey;
            D_PostEvent( &event );
            currkey = 0;
            downtime = 0;
        }
        return 0;
    }

    doomkey = xlatekey( ev.code );
    if( doomkey == 0 ) return 1;

    // same key as before?
    if( doomkey == currkey ) {
        downtime = I_GetTime();
        return 1;
    }

    // key switched without timeout?
    if( currkey != doomkey ) {
        event.type  = ev_keyup;
        event.data1 = currkey;
        D_PostEvent( &event );
    }

    // send new keypress
    event.type  = ev_keydown;
    event.data1 = doomkey;
    D_PostEvent( &event );
    currkey = doomkey;
    downtime = I_GetTime();

    return 1;

}

// This polls host keyboard events
static int I_GetEvent( void )
{
    if( kb >= 0 ) return do_kb();
    else if( ir >= 0 ) return do_ir();

    return 0;

}


//
// I_StartTic
//
void I_StartTic( void )
{
    while( I_GetEvent() )
        ;
}


//
// I_UpdateNoBlit
//
void I_UpdateNoBlit( void )
{
    // what is this?
}


// Convert from palette based picture and scale to destination

static void scale1( uint8_t *src, uint32_t *pal, int srcstride, uint32_t *dst, int dststride )
{
    int x, y;
    uint32_t dstadd;

    dstadd = dststride-SCREENWIDTH;

    for( y = 0; y < SCREENHEIGHT; y++ ) {

        for( x = 0; x < SCREENWIDTH; x += 8 ) {
            uint64_t srcw;
            srcw = __insn_ld_add( src, 8 );
            __insn_st_add( dst, __insn_v4int_l( pal[__insn_bfextu( srcw,  8, 15 )], pal[__insn_bfextu( srcw,  0,  7 )] ), 8 );
            __insn_st_add( dst, __insn_v4int_l( pal[__insn_bfextu( srcw, 24, 31 )], pal[__insn_bfextu( srcw, 16, 23 )] ), 8 );
            __insn_st_add( dst, __insn_v4int_l( pal[__insn_bfextu( srcw, 40, 47 )], pal[__insn_bfextu( srcw, 32, 39 )] ), 8 );
            __insn_st_add( dst, __insn_v4int_l( pal[__insn_bfextu( srcw, 56, 63 )], pal[__insn_bfextu( srcw, 48, 55 )] ), 8 );
        }

        dst += dstadd;

    }

}

#if 0
// 2x Average scaler
// 100%ul         50%ul/50%ur
//  50%ul/50%ll   50%ul/50%lr

//#define QUADAVG(a,b) (((a & ~0x01010101) >> 1) + ((b & ~0x01010101) >> 1) + ((a | b) & 0x01010101))
#define QUADAVG(a,b) __insn_v1avgu( a, b )

static void scale2_avg( uint8_t *src, uint32_t *pal, int srcstride, uint32_t *dst, int dststride )
{
    int x, y;
    uint8_t  *src0, *src1;
    uint32_t *dst0, *dst1;
    uint32_t dstadd;

    src0 = src; // top, use same
    src1 = src;

    dst0 =  dst;
    dst1 =  dst0 + dststride;

    dstadd = dststride*2 - SCREENWIDTH*2;

    for( y = 0; y < SCREENHEIGHT; y++ ) {
        uint32_t sul, sur, sll, slr;
        uint32_t q;

        sul = pal[src0[0]];
        sll = pal[src1[0]];

        for( x = 0; x < SCREENWIDTH-1; x++ ) {

            sur = pal[src0[x+1]];
            slr = pal[src1[x+1]];

            *dst0++ = sul;                 *dst0++ = QUADAVG( sul, sur ); // orig, right
            *dst1++ = QUADAVG( sul, sll ); *dst1++ = QUADAVG( sul, slr ); // down, diag

            sul = sur;
            sll = slr;

        }

        // last col, sur=sul,slr=sll
        sul = pal[src0[x]];
        sll = pal[src1[x]];
        q   = QUADAVG( sul, sll );
        *dst0++ = sul; *dst0++ = sul; // orig, right
        *dst1++ = q;   *dst1++ = q;   // down, diag

        src0 =  src1;
        if( y != SCREENHEIGHT-2 ) src1 += srcstride; // last row, src0=src1

        dst0 += dstadd;
        dst1 += dstadd;

    }

}
#endif

// Based on pseudocode from
// http://en.wikipedia.org/wiki/Image_scaling#EPX.2FScale2.C3.97.2FAdvMAME2.C3.97
//   A    --\ 1 2
// C P B  --/ 3 4
//   D
static void scale2_epx( uint8_t *src, uint32_t *pal, int srcstride, uint32_t *dst, int dststride )
{
    uint8_t  *src0, *src1, *src2;
    uint32_t *dst0, *dst1;
    int x, y;
    uint32_t a,b,c,d,p;
    uint32_t d1,d2,d3,d4;
    uint32_t dstadd;

    dst0 = dst;
    dst1 = dst + dststride;

    src0 = src;
    src1 = src;
    src2 = src + srcstride;

    dstadd = dststride*2-SCREENWIDTH*2;

    for( y = 0; y < SCREENHEIGHT; y++ ) {

        for( x = 0; x < SCREENWIDTH; x++ ) {

            a =                                     pal[src0[x]  ];
            c = x == 0             ? pal[src1[x]] : pal[src1[x-1]];
            p =                                     pal[src1[x]  ];
            b = x == SCREENWIDTH-1 ? pal[src1[x]] : pal[src1[x+1]];
            d =                                     pal[src2[x]  ];

            d1 = c == a && c != d && a != b ? a : p;
            d2 = a == b && a != c && b != d ? b : p;
            d4 = b == d && b != a && d != c ? d : p;
            d3 = d == c && d != b && c != a ? c : p;

            *dst0++ = d1; *dst0++ = d2;
            *dst1++ = d3; *dst1++ = d4;

        }

        dst0 += dstadd;
        dst1 += dstadd;

        src0 = src1;
        src1 = src2;
        src2 = y == SCREENHEIGHT-2 ? src1 : src2 + srcstride;
    }
}

// Based on pseudocode from
// http://en.wikipedia.org/wiki/Image_scaling#Scale3.C3.97.2FAdvMAME3.C3.97
// A B C --\  1 2 3
// D E F    > 4 5 6
// G H I --/  7 8 9
static void scale3_epx( uint8_t *src, uint32_t *pal, int srcstride, uint32_t *dst, int dststride )
{
    uint8_t  *src0, *src1, *src2;
    uint32_t *dst0, *dst1, *dst2;
    int x,y;
    uint32_t a,b,c,d,e,f,g,h,i;
    uint32_t d1,d2,d3,d4,d5,d6,d7,d8,d9;
    uint32_t dstadd;

    src0 = src;
    src1 = src;
    src2 = src + srcstride;

    dst0 = dst;
    dst1 = dst +   dststride;
    dst2 = dst + 2*dststride;

    dstadd = dststride*3-SCREENWIDTH*3;

    for( y = 0; y < SCREENHEIGHT; y++ ) {

        for( x = 0; x < SCREENWIDTH; x++ ) {
            int xl, xr;

            xl = x == 0             ? x : x-1;
            xr = x == SCREENWIDTH-1 ? x : x+1;

            a = pal[src0[xl]]; b = pal[src0[x]]; c = pal[src0[xr]];
            d = pal[src1[xl]]; e = pal[src1[x]]; f = pal[src1[xr]];
            g = pal[src2[xl]]; h = pal[src2[x]]; i = pal[src2[xr]];

            d1 =  d == b && d != h && b != f                                                        ? d : e;
            d2 = (d == b && d != h && b != f && e != c ) || (b == f && b != d && f != h && e != a)  ? b : e;
            d3 =                                             b == f && b != d && f != h             ? f : e;
            d4 = (h == d && h != f && d != b && e != a ) || (d == b && d != h && b != f && e != g ) ? d : e;
            d5 =                                                                                          e;
            d6 = (b == f && b != d && f != h && e != i ) || (f == h && f != b && h != d && e != c)  ? f : e;
            d7 =                                             h == d && h != f && d != b             ? d : e;
            d8 = (f == h && f != b && h != d && e != g ) || (h == d && h != f && d != b && e != i)  ? h : e;
            d9 =  f == h && f != b && h != d                                                        ? f : e;

            *dst0++ = d1; *dst0++ = d2; *dst0++ = d3;
            *dst1++ = d4; *dst1++ = d5; *dst1++ = d6;
            *dst2++ = d7; *dst2++ = d8; *dst2++ = d9;

        }

        dst0 += dstadd;
        dst1 += dstadd;
        dst2 += dstadd;

        src0 = src1;
        src1 = src2;
        src2 = y == SCREENHEIGHT-2 ? src1 : src2 + srcstride;
    }
}

static void scale4( uint8_t *src, uint32_t *pal, int srcstride, uint32_t *dst, int dststride )
{
    int x, y;
    uint32_t dstadd;
    uint32_t *dst0,*dst1,*dst2,*dst3;

    dst0 = dst;
    dst1 = dst +   dststride;
    dst2 = dst + 2*dststride;
    dst3 = dst + 3*dststride;

    dstadd = dststride*4-SCREENWIDTH*4;

    for( y = 0; y < SCREENHEIGHT; y++ ) {

        for( x = 0; x < SCREENWIDTH; x += 8 ) {
            uint64_t d,d00,d11,d22,d33,d44,d55,d66,d77;
            uint64_t srcw;

            srcw = __insn_ld_add( src, 8 );

            d = pal[__insn_bfextu( srcw,  0,  7 )]; d00 = __insn_v4int_l( d, d );
            d = pal[__insn_bfextu( srcw,  8, 15 )]; d11 = __insn_v4int_l( d, d );
            d = pal[__insn_bfextu( srcw, 16, 23 )]; d22 = __insn_v4int_l( d, d );
            d = pal[__insn_bfextu( srcw, 24, 31 )]; d33 = __insn_v4int_l( d, d );
            d = pal[__insn_bfextu( srcw, 32, 39 )]; d44 = __insn_v4int_l( d, d );
            d = pal[__insn_bfextu( srcw, 40, 47 )]; d55 = __insn_v4int_l( d, d );
            d = pal[__insn_bfextu( srcw, 48, 55 )]; d66 = __insn_v4int_l( d, d );
            d = pal[__insn_bfextu( srcw, 56, 63 )]; d77 = __insn_v4int_l( d, d );

            __insn_st_add( dst0, d00, 8 ); __insn_st_add( dst0, d00, 8 );
            __insn_st_add( dst0, d11, 8 ); __insn_st_add( dst0, d11, 8 );
            __insn_st_add( dst0, d22, 8 ); __insn_st_add( dst0, d22, 8 );
            __insn_st_add( dst0, d33, 8 ); __insn_st_add( dst0, d33, 8 );
            __insn_st_add( dst0, d44, 8 ); __insn_st_add( dst0, d44, 8 );
            __insn_st_add( dst0, d55, 8 ); __insn_st_add( dst0, d55, 8 );
            __insn_st_add( dst0, d66, 8 ); __insn_st_add( dst0, d66, 8 );
            __insn_st_add( dst0, d77, 8 ); __insn_st_add( dst0, d77, 8 );

            __insn_st_add( dst1, d00, 8 ); __insn_st_add( dst1, d00, 8 );
            __insn_st_add( dst1, d11, 8 ); __insn_st_add( dst1, d11, 8 );
            __insn_st_add( dst1, d22, 8 ); __insn_st_add( dst1, d22, 8 );
            __insn_st_add( dst1, d33, 8 ); __insn_st_add( dst1, d33, 8 );
            __insn_st_add( dst1, d44, 8 ); __insn_st_add( dst1, d44, 8 );
            __insn_st_add( dst1, d55, 8 ); __insn_st_add( dst1, d55, 8 );
            __insn_st_add( dst1, d66, 8 ); __insn_st_add( dst1, d66, 8 );
            __insn_st_add( dst1, d77, 8 ); __insn_st_add( dst1, d77, 8 );

            __insn_st_add( dst2, d00, 8 ); __insn_st_add( dst2, d00, 8 );
            __insn_st_add( dst2, d11, 8 ); __insn_st_add( dst2, d11, 8 );
            __insn_st_add( dst2, d22, 8 ); __insn_st_add( dst2, d22, 8 );
            __insn_st_add( dst2, d33, 8 ); __insn_st_add( dst2, d33, 8 );
            __insn_st_add( dst2, d44, 8 ); __insn_st_add( dst2, d44, 8 );
            __insn_st_add( dst2, d55, 8 ); __insn_st_add( dst2, d55, 8 );
            __insn_st_add( dst2, d66, 8 ); __insn_st_add( dst2, d66, 8 );
            __insn_st_add( dst2, d77, 8 ); __insn_st_add( dst2, d77, 8 );

            __insn_st_add( dst3, d00, 8 ); __insn_st_add( dst3, d00, 8 );
            __insn_st_add( dst3, d11, 8 ); __insn_st_add( dst3, d11, 8 );
            __insn_st_add( dst3, d22, 8 ); __insn_st_add( dst3, d22, 8 );
            __insn_st_add( dst3, d33, 8 ); __insn_st_add( dst3, d33, 8 );
            __insn_st_add( dst3, d44, 8 ); __insn_st_add( dst3, d44, 8 );
            __insn_st_add( dst3, d55, 8 ); __insn_st_add( dst3, d55, 8 );
            __insn_st_add( dst3, d66, 8 ); __insn_st_add( dst3, d66, 8 );
            __insn_st_add( dst3, d77, 8 ); __insn_st_add( dst3, d77, 8 );

        }

        dst0 += dstadd;
        dst1 += dstadd;
        dst2 += dstadd;
        dst3 += dstadd;

    }

}


static void scale5( uint8_t *src, uint32_t *pal, int srcstride, uint32_t *dst, int dststride )
{
    int x, y;
    uint32_t dstadd;
    uint32_t *dst0,*dst1,*dst2,*dst3,*dst4;

    dst0 = dst;
    dst1 = dst +   dststride;
    dst2 = dst + 2*dststride;
    dst3 = dst + 3*dststride;
    dst4 = dst + 4*dststride;

    dstadd = dststride*5-SCREENWIDTH*5;

    for( y = 0; y < SCREENHEIGHT; y++ ) {

        for( x = 0; x < SCREENWIDTH; x += 8 ) {
            uint64_t dl,dr;
            uint64_t d00,d01,d11,d22,d23,d33,d44,d45,d55,d66,d67,d77;
            uint64_t srcw;

            srcw = __insn_ld_add( src, 8 );

            dl = pal[__insn_bfextu( srcw,  0,  7 )]; d00 = __insn_v4int_l( dl, dl );
            dr = pal[__insn_bfextu( srcw,  8, 15 )]; d11 = __insn_v4int_l( dr, dr ); d01 = __insn_v4int_l( dr, dl );
            dl = pal[__insn_bfextu( srcw, 16, 23 )]; d22 = __insn_v4int_l( dl, dl );
            dr = pal[__insn_bfextu( srcw, 24, 31 )]; d33 = __insn_v4int_l( dr, dr ); d23 = __insn_v4int_l( dr, dl );
            dl = pal[__insn_bfextu( srcw, 32, 39 )]; d44 = __insn_v4int_l( dl, dl );
            dr = pal[__insn_bfextu( srcw, 40, 47 )]; d55 = __insn_v4int_l( dr, dr ); d45 = __insn_v4int_l( dr, dl );
            dl = pal[__insn_bfextu( srcw, 48, 55 )]; d66 = __insn_v4int_l( dl, dl );
            dr = pal[__insn_bfextu( srcw, 56, 63 )]; d77 = __insn_v4int_l( dr, dr ); d67 = __insn_v4int_l( dr, dl );

            __insn_st_add( dst0, d00, 8 ); __insn_st_add( dst0, d00, 8 ); __insn_st_add( dst0, d01, 8 );
            __insn_st_add( dst0, d11, 8 ); __insn_st_add( dst0, d11, 8 );
            __insn_st_add( dst0, d22, 8 ); __insn_st_add( dst0, d22, 8 ); __insn_st_add( dst0, d23, 8 );
            __insn_st_add( dst0, d33, 8 ); __insn_st_add( dst0, d33, 8 );
            __insn_st_add( dst0, d44, 8 ); __insn_st_add( dst0, d44, 8 ); __insn_st_add( dst0, d45, 8 );
            __insn_st_add( dst0, d55, 8 ); __insn_st_add( dst0, d55, 8 );
            __insn_st_add( dst0, d66, 8 ); __insn_st_add( dst0, d66, 8 ); __insn_st_add( dst0, d67, 8 );
            __insn_st_add( dst0, d77, 8 ); __insn_st_add( dst0, d77, 8 );

            __insn_st_add( dst1, d00, 8 ); __insn_st_add( dst1, d00, 8 ); __insn_st_add( dst1, d01, 8 );
            __insn_st_add( dst1, d11, 8 ); __insn_st_add( dst1, d11, 8 );
            __insn_st_add( dst1, d22, 8 ); __insn_st_add( dst1, d22, 8 ); __insn_st_add( dst1, d23, 8 );
            __insn_st_add( dst1, d33, 8 ); __insn_st_add( dst1, d33, 8 );
            __insn_st_add( dst1, d44, 8 ); __insn_st_add( dst1, d44, 8 ); __insn_st_add( dst1, d45, 8 );
            __insn_st_add( dst1, d55, 8 ); __insn_st_add( dst1, d55, 8 );
            __insn_st_add( dst1, d66, 8 ); __insn_st_add( dst1, d66, 8 ); __insn_st_add( dst1, d67, 8 );
            __insn_st_add( dst1, d77, 8 ); __insn_st_add( dst1, d77, 8 );

            __insn_st_add( dst2, d00, 8 ); __insn_st_add( dst2, d00, 8 ); __insn_st_add( dst2, d01, 8 );
            __insn_st_add( dst2, d11, 8 ); __insn_st_add( dst2, d11, 8 );
            __insn_st_add( dst2, d22, 8 ); __insn_st_add( dst2, d22, 8 ); __insn_st_add( dst2, d23, 8 );
            __insn_st_add( dst2, d33, 8 ); __insn_st_add( dst2, d33, 8 );
            __insn_st_add( dst2, d44, 8 ); __insn_st_add( dst2, d44, 8 ); __insn_st_add( dst2, d45, 8 );
            __insn_st_add( dst2, d55, 8 ); __insn_st_add( dst2, d55, 8 );
            __insn_st_add( dst2, d66, 8 ); __insn_st_add( dst2, d66, 8 ); __insn_st_add( dst2, d67, 8 );
            __insn_st_add( dst2, d77, 8 ); __insn_st_add( dst2, d77, 8 );

            __insn_st_add( dst3, d00, 8 ); __insn_st_add( dst3, d00, 8 ); __insn_st_add( dst3, d01, 8 );
            __insn_st_add( dst3, d11, 8 ); __insn_st_add( dst3, d11, 8 );
            __insn_st_add( dst3, d22, 8 ); __insn_st_add( dst3, d22, 8 ); __insn_st_add( dst3, d23, 8 );
            __insn_st_add( dst3, d33, 8 ); __insn_st_add( dst3, d33, 8 );
            __insn_st_add( dst3, d44, 8 ); __insn_st_add( dst3, d44, 8 ); __insn_st_add( dst3, d45, 8 );
            __insn_st_add( dst3, d55, 8 ); __insn_st_add( dst3, d55, 8 );
            __insn_st_add( dst3, d66, 8 ); __insn_st_add( dst3, d66, 8 ); __insn_st_add( dst3, d67, 8 );
            __insn_st_add( dst3, d77, 8 ); __insn_st_add( dst3, d77, 8 );

            __insn_st_add( dst4, d00, 8 ); __insn_st_add( dst4, d00, 8 ); __insn_st_add( dst4, d01, 8 );
            __insn_st_add( dst4, d11, 8 ); __insn_st_add( dst4, d11, 8 );
            __insn_st_add( dst4, d22, 8 ); __insn_st_add( dst4, d22, 8 ); __insn_st_add( dst4, d23, 8 );
            __insn_st_add( dst4, d33, 8 ); __insn_st_add( dst4, d33, 8 );
            __insn_st_add( dst4, d44, 8 ); __insn_st_add( dst4, d44, 8 ); __insn_st_add( dst4, d45, 8 );
            __insn_st_add( dst4, d55, 8 ); __insn_st_add( dst4, d55, 8 );
            __insn_st_add( dst4, d66, 8 ); __insn_st_add( dst4, d66, 8 ); __insn_st_add( dst4, d67, 8 );
            __insn_st_add( dst4, d77, 8 ); __insn_st_add( dst4, d77, 8 );

        }

        dst0 += dstadd;
        dst1 += dstadd;
        dst2 += dstadd;
        dst3 += dstadd;
        dst4 += dstadd;

    }

}

// This should never be used since we don't support 1920x1200.
// If 1920x1200 support is added, make a custom x6 scaler, please.
static void scale_fact( uint8_t *src, uint32_t *pal, int srcstride, uint32_t *dst, int dststride, int fact )
{
    int x, y, xx, yy;

    for( y = 0; y < SCREENHEIGHT; y++ ) {

        for( x = 0; x < SCREENWIDTH*fact; x += fact ) {

            uint32_t col = pal[*src++];
            uint32_t *dstrow = dst;

            for( yy = 0; yy < fact; yy++ ) {

                for( xx = 0; xx < fact; xx++ )  {

                    dstrow[x+xx] = col;

                }

                dstrow += dststride;

            }

        }

        dst += dststride*fact;

    }

}

//
// I_FinishUpdate
//
void I_FinishUpdate( void )
{
    // draws little dots on the bottom of the screen
//    if( devparm ) {
    if( true ) {
        int i;
        int tics;
        static int lasttic;

        i = I_GetTime();
        tics = i - lasttic;
        lasttic = i;
        if( tics > 20 ) tics = 20;

        for( i = 0 ; i < tics*2; i += 2 )
            screens[0][(SCREENHEIGHT-1)*SCREENWIDTH + i] = 0xff;
        for ( ; i < 20*2 ; i+=2 )
            screens[0][(SCREENHEIGHT-1)*SCREENWIDTH + i] = 0x00;
    }

    // Convert from 8 bit to 32 bit rgba and scale.
    switch( multiply ) {
    case 1:
        scale1( screens[0], pal32, SCREENWIDTH, myscreen, mystride );
        break;

    case 2:
//        scale2_avg( screens[0], SCREENWIDTH, myscreen, mystride );
        scale2_epx( screens[0], pal32, SCREENWIDTH, myscreen, mystride );
        break;

    case 3:
        scale3_epx( screens[0], pal32, SCREENWIDTH, myscreen, mystride );
        break;

    case 4:
        scale4( screens[0], pal32, SCREENWIDTH, myscreen, mystride );
        break;

    case 5:
        scale5( screens[0], pal32, SCREENWIDTH, myscreen, mystride );
        break;

    default:
        scale_fact( screens[0], pal32, SCREENWIDTH, myscreen, mystride, multiply );
        break;

    }

    if( !measure )
       tmc_sync_barrier_wait( bar );

}


//
// I_ReadScreen
//
void I_ReadScreen( byte *scr )
{
    memcpy( scr, screens[0], SCREENWIDTH*SCREENHEIGHT );
}


//
// I_SetPalette
//

void I_SetPalette( byte *palette )
{
    int i;

    // gamma table 0 is just baloney
    if( !usegamma ) {
        for( i = 0; i < 256; ++i ) {
            uint64_t r,g,b;
            r = *palette++;
            g = *palette++;
            b = *palette++;
            pal32[i] = __insn_v2int_l( __insn_v1int_l( 0, r ), __insn_v1int_l( g, b ) );
        }
    } else {
        for( i = 0; i < 256; ++i ) {
            uint64_t r,g,b;
            r = gammatable[usegamma][*palette++];
            g = gammatable[usegamma][*palette++];
            b = gammatable[usegamma][*palette++];
            pal32[i] = __insn_v2int_l( __insn_v1int_l( 0, r ), __insn_v1int_l( g, b ) );
        }
    }
}

void I_InitGraphics( void )
{
    static int firsttime = 1;

    if( !firsttime ) return;
    firsttime = 0;

    if( M_CheckParm( "-2" ) ) multiply = 2;
    if( M_CheckParm( "-3" ) ) multiply = 3;

    // check if the user wants to grab the mouse (quite unnice)
    grabMouse = !!M_CheckParm( "-grabmouse" );

    screens[0] = (unsigned char *)malloc( SCREENWIDTH * SCREENHEIGHT );
    if( screens[0] == NULL )
        I_Error( "Couldn't allocate screen memory" );

    // insert a delay before starting a core since demo runs on a 6 or 7 step loop.
    // this will keep them sufficiently out of sync.
    if( !measure ) {
        int i, val;
        val = 8 * coreid;
        for( i = 0; i < val; i++ ) {
            tmc_sync_barrier_wait( bar );
            tmc_sync_barrier_wait( bar );
        }
    }

    printf( "%d: Ready\n", coreid );

}
